@@ -74,17 +74,17 @@ extern crate rental;
 pub use crate::ast::query::SelectType;
 pub use crate::ctx::{EventContext, EventOriginUri};
 use crate::prelude::*;
-pub use crate::query::Query;
+pub use crate::query::{Query, QueryRental};
 pub use crate::registry::{
     aggr as aggr_registry, registry, Aggr as AggrRegistry, CustomFn, Registry, TremorAggrFn,
     TremorAggrFnWrapper, TremorFn, TremorFnWrapper,
 };
 pub use crate::script::{Return, Script};
-use lazy_static::lazy_static;
-use std::sync::atomic::{AtomicU32, Ordering};
-
 use ast::{Consts, InvokeAggrFn};
 pub use interpreter::{AggrType, FALSE, NULL, TRUE};
+use lazy_static::lazy_static;
+use std::sync::atomic::{AtomicU32, Ordering};
+use std::{mem, pin::Pin};
 pub use tremor_common::stry;
 pub use tremor_value::{KnownKey, Object, Value};
 
@@ -139,15 +139,7 @@ impl<'event> ValueAndMeta<'event> {
     pub fn value(&self) -> &Value<'event> {
         &self.v
     }
-    /// Event value forced to borrowd mutable (uses `mem::transmute`)
-    ///
-    /// # Safety
-    /// This isn't save, use with care and reason about mutability!
-    #[allow(mutable_transmutes, clippy::transmute_ptr_to_ptr, clippy::mut_from_ref)]
-    #[must_use]
-    pub unsafe fn force_value_mut(&self) -> &mut Value<'event> {
-        std::mem::transmute(&self.v)
-    }
+
     /// Event value
     #[must_use]
     pub fn value_mut(&mut self) -> &mut Value<'event> {
@@ -163,6 +155,11 @@ impl<'event> ValueAndMeta<'event> {
     pub fn into_parts(self) -> (Value<'event>, Value<'event>) {
         (self.v, self.m)
     }
+    /// borrows both parts as mutalbe
+    #[must_use]
+    pub fn parts_mut(&mut self) -> (&mut Value<'event>, &mut Value<'event>) {
+        (&mut self.v, &mut self.m)
+    }
 }
 
 impl<'event> Default for ValueAndMeta<'event> {
@@ -188,12 +185,13 @@ rental! {
     /// issues
     pub(crate) mod rentals {
         use super::*;
+        use std::pin::Pin;
 
         /// Rental wrapped value with the data it was parsed
         /// from from
         #[rental_mut(covariant, debug)]
         pub struct Value {
-            raw: Vec<Vec<u8>>,
+            raw: Vec<Pin<Vec<u8>>>,
             parsed: ValueAndMeta<'raw>
         }
 
@@ -202,23 +200,15 @@ rental! {
 
 impl rentals::Value {
     /// Borrow the parts (event and metadata) from a rental.
-    /// This borrows the data as immutable and then transmutes it
-    /// to be mutable.
-    #[allow(mutable_transmutes, clippy::transmute_ptr_to_ptr)]
     #[must_use]
-    pub fn parts<'value, 'borrow>(
-        &'borrow self,
-    ) -> (&'borrow mut Value<'value>, &'borrow mut Value<'value>)
+    pub fn parts<'value, 'borrow>(&'borrow self) -> (&'borrow Value<'value>, &'borrow Value<'value>)
     where
         'borrow: 'value,
     {
-        unsafe {
-            let data = self.suffix();
-            let unwind_event: &'borrow mut Value<'value> = std::mem::transmute(data.value());
-            let event_meta: &'borrow mut Value<'value> = std::mem::transmute(data.meta());
-            (unwind_event, event_meta)
-        }
+        let ValueAndMeta { v, m } = self.suffix();
+        (v, m)
     }
+
     /// Consumes an event into another
     /// This function works around a rental limitation that is meant
     /// to protect its users: Rental does not allow you to get both
@@ -250,13 +240,15 @@ impl rentals::Value {
     {
         struct ScrewRental {
             pub parsed: ValueAndMeta<'static>,
-            pub raw: Vec<Vec<u8>>,
+            pub raw: Vec<Pin<Vec<u8>>>,
         }
+        // ALLOW: https://github.com/tremor-rs/tremor-runtime/issues/1031
         #[allow(clippy::transmute_ptr_to_ptr)]
         unsafe {
-            use std::mem::transmute;
-            let self_unrent: &'run mut ScrewRental = transmute(self);
-            let mut other_unrent: ScrewRental = transmute(other);
+            // ALLOW: https://github.com/tremor-rs/tremor-runtime/issues/1031
+            let self_unrent: &'run mut ScrewRental = mem::transmute(self);
+            // ALLOW: https://github.com/tremor-rs/tremor-runtime/issues/1031
+            let mut other_unrent: ScrewRental = mem::transmute(other);
             self_unrent.raw.append(&mut other_unrent.raw);
             join_f(&mut self_unrent.parsed, other_unrent.parsed)?;
         }
